@@ -29,10 +29,13 @@ import inspect
 from ..overrides import override
 from ..importer import modules
 
+from gi.repository import GLib
+
 
 if sys.version_info >= (3, 0):
     _basestring = str
     _callable = lambda c: hasattr(c, '__call__')
+    long = int
 else:
     _basestring = basestring
     _callable = callable
@@ -68,15 +71,24 @@ class Caps(Gst.Caps):
     def __nonzero__(self):
         return not self.is_empty()
 
-    def __new__(cls, *kwargs):
-        if not kwargs:
+    def __new__(cls, *args):
+        if not args:
             return Caps.new_empty()
-        elif len(kwargs) > 1:
+        elif len(args) > 1:
             raise TypeError("wrong arguments when creating GstCaps object")
-        elif isinstance(kwargs[0], str):
-            return Caps.from_string(kwargs[0])
-        elif isinstance(kwargs[0], Caps):
-            return kwargs[0].copy()
+        elif isinstance(args[0], str):
+            return Caps.from_string(args[0])
+        elif isinstance(args[0], Caps):
+            return args[0].copy()
+        elif isinstance(args[0], Structure):
+            res = Caps.new_empty()
+            res.append_structure(args[0])
+            return res
+        elif isinstance(args[0], (list, tuple)):
+            res = Caps.new_empty()
+            for e in args[0]:
+                res.append_structure(e)
+            return res
 
         raise TypeError("wrong arguments when creating GstCaps object")
 
@@ -224,12 +236,48 @@ Pipeline = override(Pipeline)
 __all__.append('Pipeline')
 
 class Structure(Gst.Structure):
+    def __new__(cls, *args, **kwargs):
+        if not args:
+            if kwargs:
+                raise TypeError("wrong arguments when creating GstStructure, first argument"
+                                " must be the structure name.")
+            return Structure.new_empty()
+        elif len(args) > 1:
+            raise TypeError("wrong arguments when creating GstStructure object")
+        elif isinstance(args[0], str):
+            if not kwargs:
+                return Structure.from_string(args[0])[0]
+            struct = Structure.new_empty(args[0])
+            for k, v in kwargs.items():
+                struct[k] = v
+
+            return struct
+        elif isinstance(args[0], Structure):
+            return args[0].copy()
+
+        raise TypeError("wrong arguments when creating GstStructure object")
+
+    def __init__(self, *args, **kwargs):
+        pass
+
     def __getitem__(self, key):
         return self.get_value(key)
 
+    def keys(self):
+        keys = set()
+        def foreach(fid, value, unused1, udata):
+            keys.add(GLib.quark_to_string(fid))
+            return True
+
+        self.foreach(foreach, None, None)
+        return keys
+
     def __setitem__(self, key, value):
         return self.set_value(key, value)
 
+    def __str__(self):
+        return self.to_string()
+
 Structure = override(Structure)
 __all__.append('Structure')
 
@@ -269,7 +317,7 @@ class Fraction(Gst.Fraction):
         self.type = "fraction"
 
     def __repr__(self):
-        return '<Gst.Fraction %d/%d>' % (self.num, self.denom)
+        return '<Gst.Fraction %s>' % (str(self))
 
     def __value__(self):
         return self.num / self.denom
@@ -288,7 +336,8 @@ class Fraction(Gst.Fraction):
                             self.denom * other.denom)
         elif isinstance(other, int):
             return Fraction(self.num * other, self.denom)
-        raise TypeError
+        raise TypeError("%s is not supported, use Gst.Fraction or int." %
+                (type(other)))
 
     __rmul__ = __mul__
 
@@ -298,23 +347,214 @@ class Fraction(Gst.Fraction):
                             self.denom * other.num)
         elif isinstance(other, int):
             return Fraction(self.num, self.denom * other)
-        return TypeError
+        return TypeError("%s is not supported, use Gst.Fraction or int." %
+                (type(other)))
 
     __div__ = __truediv__
 
     def __rtruediv__(self, other):
         if isinstance(other, int):
             return Fraction(self.denom * other, self.num)
-        return TypeError
+        return TypeError("%s is not an int." % (type(other)))
 
     __rdiv__ = __rtruediv__
 
     def __float__(self):
         return float(self.num) / float(self.denom)
 
+    def __str__(self):
+        return '%d/%d' % (self.num, self.denom)
+
 Fraction = override(Fraction)
 __all__.append('Fraction')
 
+
+class IntRange(Gst.IntRange):
+    def __init__(self, r):
+        if not isinstance(r, range):
+            raise TypeError("%s is not a range." % (type(r)))
+
+        if (r.start >= r.stop):
+            raise TypeError("Range start must be smaller then stop")
+
+        if r.start % r.step != 0:
+            raise TypeError("Range start must be a multiple of the step")
+
+        if r.stop % r.step != 0:
+            raise TypeError("Range stop must be a multiple of the step")
+
+        self.range = r
+
+    def __repr__(self):
+        return '<Gst.IntRange [%d,%d,%d]>' % (self.range.start,
+                self.range.stop, self.range.step)
+
+    def __str__(self):
+        if self.range.step == 1:
+            return '[%d,%d]' % (self.range.start, self.range.stop)
+        else:
+            return '[%d,%d,%d]' % (self.range.start, self.range.stop,
+                    self.range.step)
+
+    def __eq__(self, other):
+        if isinstance(other, range):
+            return self.range == other
+        elif isinstance(other, IntRange):
+            return self.range == other.range
+        return False
+
+if sys.version_info >= (3, 0):
+    IntRange = override(IntRange)
+    __all__.append('IntRange')
+
+
+class Int64Range(Gst.Int64Range):
+    def __init__(self, r):
+        if not isinstance(r, range):
+            raise TypeError("%s is not a range." % (type(r)))
+
+        if (r.start >= r.stop):
+            raise TypeError("Range start must be smaller then stop")
+
+        if r.start % r.step != 0:
+            raise TypeError("Range start must be a multiple of the step")
+
+        if r.stop % r.step != 0:
+            raise TypeError("Range stop must be a multiple of the step")
+
+        self.range = r
+
+    def __repr__(self):
+        return '<Gst.Int64Range [%d,%d,%d]>' % (self.range.start,
+                self.range.stop, self.range.step)
+
+    def __str__(self):
+        if self.range.step == 1:
+            return '(int64)[%d,%d]' % (self.range.start, self.range.stop)
+        else:
+            return '(int64)[%d,%d,%d]' % (self.range.start, self.range.stop,
+                    self.range.step)
+
+    def __eq__(self, other):
+        if isinstance(other, range):
+            return self.range == other
+        elif isinstance(other, IntRange):
+            return self.range == other.range
+        return False
+
+class Bitmask(Gst.Bitmask):
+    def __init__(self, v):
+        if not isinstance(v, long) and not isinstance(v, int):
+            raise TypeError("%s is not an int or long." % (type(v)))
+
+        self.v = long(v)
+
+    def __str__(self):
+        return hex(self.v)
+
+    def __eq__(self, other):
+        return self.v == other
+
+
+Bitmask = override(Bitmask)
+__all__.append('Bitmask')
+
+
+if sys.version_info >= (3, 0):
+    Int64Range = override(Int64Range)
+    __all__.append('Int64Range')
+
+
+class DoubleRange(Gst.DoubleRange):
+    def __init__(self, start, stop):
+        self.start = float(start)
+        self.stop = float(stop)
+
+        if (start >= stop):
+            raise TypeError("Range start must be smaller then stop")
+
+    def __repr__(self):
+        return '<Gst.DoubleRange [%s,%s]>' % (str(self.start), str(self.stop))
+
+    def __str__(self):
+        return '(double)[%s,%s]' % (str(self.range.start), str(self.range.stop))
+
+
+DoubleRange = override(DoubleRange)
+__all__.append('DoubleRange')
+
+
+class FractionRange(Gst.FractionRange):
+    def __init__(self, start, stop):
+        if not isinstance(start, Gst.Fraction):
+            raise TypeError("%s is not a Gst.Fraction." % (type(start)))
+
+        if not isinstance(stop, Gst.Fraction):
+            raise TypeError("%s is not a Gst.Fraction." % (type(stop)))
+
+        if (float(start) >= float(stop)):
+            raise TypeError("Range start must be smaller then stop")
+
+        self.start = start
+        self.stop = stop
+
+    def __repr__(self):
+        return '<Gst.FractionRange [%s,%s]>' % (str(self.start),
+                str(self.stop))
+
+    def __str__(self):
+        return '(fraction)[%s,%s]' % (str(self.start), str(self.stop))
+
+FractionRange = override(FractionRange)
+__all__.append('FractionRange')
+
+
+class ValueArray(Gst.ValueArray):
+    def __init__(self, array):
+        self.array = list(array)
+
+    def __getitem__(self, index):
+        return self.array[index]
+
+    def __setitem__(self, index, value):
+        self.array[index] = value
+
+    def __len__(self):
+        return len(self.array)
+
+    def __str__(self):
+        return '<' + ','.join(map(str,self.array)) + '>'
+
+    def __repr__(self):
+        return '<Gst.ValueArray %s>' % (str(self))
+
+ValueArray = override(ValueArray)
+__all__.append('ValueArray')
+
+
+class ValueList(Gst.ValueList):
+    def __init__(self, array):
+        self.array = list(array)
+
+    def __getitem__(self, index):
+        return self.array[index]
+
+    def __setitem__(self, index, value):
+        self.array[index] = value
+
+    def __len__(self):
+        return len(self.array)
+
+    def __str__(self):
+        return '{' + ','.join(map(str,self.array)) + '}'
+
+    def __repr__(self):
+        return '<Gst.ValueList %s>' % (str(self))
+
+ValueList = override(ValueList)
+__all__.append('ValueList')
+
+
 def TIME_ARGS(time):
     if time == Gst.CLOCK_TIME_NONE:
         return "CLOCK_TIME_NONE"
@@ -394,4 +634,5 @@ def deinit():
 
 Gst.deinit = deinit
 
-deinit_pygst()
+if not Gst.is_initialized():
+    deinit_pygst()
